
package XSA;

import java.io.*;
import java.util.Hashtable;
import java.net.URL;
import java.net.URLConnection;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * The main XSA object, which controls the whole checking process and presents
 * an API to clients.
 */

public class XSAEngine {
  private XSAClient     client;
  private String        parserDriver;
  private ObjectFactory factory;
  private InternalObjectsFactory internalFactory;

  public XSAEngine() {
    factory=new ObjectFactory();
    internalFactory=new InternalObjectsFactory();
  }

  /**
   * Registers a new ObjectFactory for use by the XSAEngine.
   */

  public void setFactory(ObjectFactory newFact) {
    factory=newFact;
  }

  /**
   * Registers a new InternalObjectsFactory for use by the engine.
   */

  public void setInternalFactory(InternalObjectsFactory newFact) {
    internalFactory=newFact;
  }
  
  /**
   * Registers the XSA client application. This method can be called at any
   * time.
   * @param _client A client object to give results and error messages.
   * @see XSAClient
   */

  public void setClient(XSAClient _client) {
    client=_client;
  }

  /**
   * Tells the engine which SAX parser to use. This method can be called at
   * any time.
   * @param className The class name of the SAX driver to use.
   */

  public void setParser(String className) {
    parserDriver=className;
  }

  /**
   * Tells the engine that checking is about to begin.
   */

  public void checkingStarts() {
    client.checkingStarts();
  }
  
  /**
   * Tells the engine to check a URL and some of the products mentioned there.
   * This method does not return until the URL has been checked.
   * @param url The URL of the XSA document to check.
   * @param pids A hashtable where the IDs of the products to check are keys.
                 (The values mapped to by the keys are never looked at.) Only
		 products whose IDs are keys in the hashtable will be checked.
		 If the parameter is null all products will be checked.
   */

  public void checkURL(String url, Hashtable pids) {
    try {
      client.checkedVendor(loadDocument(url,pids));
    }
    catch (InternalXSAException e) {
      client.error(url,e.getMessage());
    }
  }

  /**
   * Parses an XSA document into a Vendor object and returns the Vendor
   * object.
   * @param url The URL to the document.
   * @param pids A hashtable of the IDs of the products to include in the
   *             object. Objects with IDs that are not in the table will
   *             be ignored. If the parameter is null all products will be
   *             included.
   * @exception InternalXSAException Thrown if anything goes wrong.
   */
  
  public Vendor loadDocument(String url, Hashtable pids) throws InternalXSAException {
    try {
      Parser parser=internalFactory.createParser(parserDriver);
      XSADocumentHandler handler=
	internalFactory.createDocHandler(parser,client,url,pids,factory);
      parser.setDocumentHandler(handler);
      parser.setErrorHandler(internalFactory.createErrorHandler(url,client));

      InputSource is=new InputSource(url);
      URL parsedUrl;
      try {
	parsedUrl=new URL(url);
	URLConnection uc=parsedUrl.openConnection();

	if (parsedUrl.getProtocol().equalsIgnoreCase("http")) {
	  // Has the XSA document moved?
	  HttpURLConnection huc=(HttpURLConnection) uc;
	  huc.setFollowRedirects(false);
	  if (huc.getResponseCode()==301 || huc.getResponseCode()==302) {
	    // It's moved. Notify client and try new URL.
	    String newURL=huc.getHeaderField("Location");
	    client.documentMoved(url,newURL);
	      
	    is=new InputSource(newURL);
	    parsedUrl=new URL(newURL);
	    uc=parsedUrl.openConnection();
	  }
	}

	is.setByteStream(uc.getInputStream());
      }
      catch (MalformedURLException e) {
	is.setCharacterStream(new FileReader(url));
      }		
	
      parser.parse(is);
      return handler.getVendor();
    }
    catch (SAXException e) {
      if (e.getMessage()!=null)
	throw new InternalXSAException("SAXException: "+e.getMessage());
      else if (e.getException()!=null) {
	e.getException().printStackTrace();
	throw new InternalXSAException("SAXException: "+e.getException().toString());
      } else 
	throw new InternalXSAException("Unknown SAX exception.");
    }
    catch (IOException e) {
      throw new InternalXSAException("IOException: "+e.getMessage());
    }
    catch (Exception e) {
      throw new InternalXSAException(e.toString());
    }
  }
  
  /**
   * Tells the engine that no more URLs will be checked.
   */

  public void checkingDone() {
    client.checkingDone();
  }

  /**
   * Returns the version number of the XSA engine implementation. This
   * number may differ from the API version because of bug fixes.
   * @return The engine version number as a string of the form "1.00",
   *         "1.10" ...
   */

  public String getEngineVersion() {
    return "1.00";
  }

  /**
   * Returns the version number of the XSA API.
   * @return The API version number as a string of the form "1.00", "1.10" ...
   */

  public String getAPIVersion() {
    return "1.00";
  }

  /**
   * Returns the version number of the XSA specification that this XSA API
   * implementation conforms to.
   * @return The XSA version number as a string of the form "1.0", "1.1" ...
   */

  public String getXSAVersion() {
    return "1.0";
  }

  /**
   * Turns the file reference into an absolute file: URL. The actual
   * implementation is taken from James Clark's XMLTest.
   * @param fileref An absolute or relative file reference.
   */

  public String fileToURL(String fileref) {
    File   file = new File(fileref);
    String path = file.getAbsolutePath();
    String fSep = System.getProperty("file.separator");
    if (fSep != null && fSep.length() == 1)
      path = path.replace(fSep.charAt(0), '/');
    if (path.length() > 0 && path.charAt(0) != '/')
      path = '/' + path;
    try {
      return new URL("file", null, path).toString();
    }
    catch (java.net.MalformedURLException e) {
      /* According to the spec this could only happen if the file
	 protocol were not recognized. */
      throw new Error("Unexpected MalformedURLException");
    }
  }

  // --- Internal methods
  
}
